/* pvc_image_gray.c
 * 
 * 
 * Copyright (C) 2012 Ivo Ihrke
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include <iostream>
#include <limits>

#include "../Image/pvc_image.h"

#include "../Image/pvc_image_gray.h"

using namespace std;


PVC_Image_Gray::PVC_Image_Gray()
{
  data = NULL;
  alignment_offset = 0;

  width = 0;
  height = 0;
}


PVC_Image_Gray::PVC_Image_Gray( int img_width, int img_height )
{
  /* memory for gray data */
  alloc_mem( width * height );

  width = img_width;
  height = img_height;
 
}

PVC_Image_Gray::PVC_Image_Gray(PVC_Image* color)
{
	data= NULL;
	width= color->w();
	height= color->h();
	
	cerr << " starting alloc" << endl;

	/* memory for gray data */
	alloc_mem( width * height );
	
	cerr << " end alloc" << endl;

	float* colorData= color->get_data();

	for(int i= 0; i<width*height; i++)
	{
		data[i]= (colorData[i*4] + colorData[i*4+1] + colorData[i*4+2])/3.0f;
	}
	
}

PVC_Image_Gray::~PVC_Image_Gray()
{
  if ( data != NULL )
    {
      /* get the original data pointer */
      data = (float *) ( (unsigned char *)data - alignment_offset );

      /* free it */
      delete [] data;
    }
}


int PVC_Image_Gray::resize( int img_width, int img_height, int mode )
{
  /* remove old image */
  if ( data != NULL )
    {
      /* get the original dat pointer */
      data -= alignment_offset;

      /* free it */
      delete [] data;
    }

  /* memory for gray data */
  alloc_mem( img_width * img_height , mode );

  width = img_width;
  height = img_height;
  
  return 0;
}



/* allocates memory in data */
int PVC_Image_Gray::alloc_mem( unsigned long int num_bytes, int mode )
{
  if( data != NULL )
    delete [] data;

  data = new float [ num_bytes + 15 ];

  if ( mode == PVC_IMAGE_ALIGNED_MEM )
    // the last % 16 is for the case that the memory is already aligned 
    alignment_offset = (16 - (unsigned long int)data % 16) % 16; 

  if ( mode == PVC_IMAGE_UNALIGNED_MEM )
    alignment_offset = 0;
 
  data = (float *)((unsigned char*)data + alignment_offset);

#ifdef DEBUG
  fprintf( stderr,"PVC_Image_Gray::alloc_mem: Memory is aligned at %d\n", (unsigned long int)data % 16 );
#endif
  
  return 0;
}

int PVC_Image_Gray::save(const char* filename)
{
	PVC_Image* color= new PVC_Image();
	color->resize(width,height);
	float* colorData= color->get_data();

	int j= 0;
	for(int i= 0; i<width*height; i++)
	{
		for(int c= 0; c<3; c++)
		{
			colorData[j++]= data[i];
		}
		j++;
	}
	int retVal= color->save(filename);
	delete color;
	return retVal;
}


void PVC_Image_Gray::normalize()
{
	float* data= get_data();
	float minVal= numeric_limits<float>::max();
	float maxVal= numeric_limits<float>::min();

	for (int i= 0; i<w()*h(); i++)
	{
		if (data[i]<minVal)
			minVal= data[i];
		if (data[i]>maxVal)
			maxVal= data[i];
	}
	for (int i= 0; i<w()*h(); i++)
	{
		const float value= data[i];
		data[i]= (value-minVal)/(maxVal-minVal);
	}
}

